#include <sys/times.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <math.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_cdf.h>

#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/boyer_myrvold_planar_test.hpp>

using namespace boost;

typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS, boost::property<vertex_index_t, int> > Graph;
typedef graph_traits<Graph>::edge_iterator EdgeIt;
typedef graph_traits<Graph>::out_edge_iterator OutEdgeIt;
typedef graph_traits<Graph>::vertex_iterator VertexIt;
typedef graph_traits<Graph>::edge_descriptor EdgeD;

template<typename Graph, typename EdgeIt, typename VertexIt>
void g2Dot(Graph g)
//void g2Dot(Graph g, std::string filename)
{
//  std::ofstream fout(filename.c_str());
  std::cerr << "graph G {" << std::endl;
  std::cerr << "graph[size=\"100,100\"];" << std::endl;

  // var nodes 
  std::cerr << "node[shape=circle,width=0.25,height=0.25,fixedsize=true];" << std::endl;
	VertexIt vi, ve;
	for (tie(vi, ve) = vertices(g); vi != ve; vi++) {
    std::cerr << "\t" << *vi;
    std::cerr << " [style=\"setlinewidth(1)\",label=" << *vi << "];" << std::endl;
  }

	// edges 
  EdgeIt ei, ei_end;
  for (tie(ei, ei_end) = edges(g); ei != ei_end; ei++) {
		std::cerr << "\t" << source(*ei, g) << " -- " << target(*ei, g);
		std::cerr << "[style=\"setlinewidth(1)\",fontsize=15,label=\"";
		std::cerr << "\"];" << std::endl;
  }
  std::cerr << "}" << std::endl;
//  std::cerr.close();
}

//void writeFg(const Graph &g, std::string filename, gsl_rng *r, double sigma, double theta)
void writeFg(const Graph &g, gsl_rng *r, double sigma, double theta, int type)
{
	std::cout << std::setprecision(14) << num_edges(g) + num_vertices(g) << std::endl << std::endl;// + num_vertices(g)

	// pairwise potentials
	VertexIt vb, ve;
	for (tie(vb, ve) = vertices(g); vb != ve; vb++) {
		OutEdgeIt eb, ee;
		for (tie(eb, ee) = out_edges(*vb, g); eb != ee; eb++) {

			if (*vb < target(*eb, g)) {

				std::cout << 2 << std::endl;
				std::cout << *vb << " " << target(*eb, g) << std::endl;
				std::cout << "2 2" << std::endl<< 4 << std::endl;
		
				double w = gsl_ran_gaussian(r, sigma);
				if (type == 0) w = fabs(w);

//				std::cout << w << std::endl;
		
				std::cout << "0\t" << exp(w) << std::endl;
				std::cout << "1\t" << exp(-w) << std::endl;
				std::cout << "2\t" << exp(-w) << std::endl;
				std::cout << "3\t" << exp(w) << std::endl << std::endl;
			}
		}

		// single variable potential
		std::cout << 1 << std::endl;
		std::cout << *vb << std::endl;
		std::cout << "2" << std::endl<< 2 << std::endl;
		
		double w = gsl_ran_gaussian(r, theta);
		if (type == 0) w = fabs(w);

		std::cout << "0\t" << exp(w) << std::endl;
		std::cout << "1\t" << exp(-w) << std::endl << std::endl;
	}
//	std::cout.close();
}

int main( int argc, char *argv[] ) {
	int N, seed, type;
	double sigma, theta;

	if (argc != 6) {
		std::cerr << "./create_grid type seed N theta sigma" << std::endl;
		exit(-1);
	}

	type = atoi(argv[1]);
	seed = atoi(argv[2]);
	N = atoi(argv[3]);
	theta = atof(argv[4]);
	sigma = atof(argv[5]);

	std::cout << "# type (0 fe, 1 sg) " << type << std::endl;
	std::cout << "# seed      " << seed << std::endl;
	std::cout << "# NxN ( N = " << N << " )" << std::endl;
	std::cout << "# theta     " << theta << std::endl;
	std::cout << "# sigma     " << sigma << std::endl;
	std::cout << "# ----------" << std::endl;
	
	// Init pseudorandom generator
  gsl_rng *r = gsl_rng_alloc(gsl_rng_mrg);
	gsl_rng_set(r, seed);					// set the rand seed

	// create spider web (this makes the graph planar when converted to Forney)
	Graph g;
	int v = 0;
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			if (j < (N-1)) add_edge(v, v+1, g);
			if (i < (N-1)) add_edge(v, v+N, g);
			v++;
		}
	}

	if (boyer_myrvold_planarity_test(g)) {
	
		writeFg(g, r, sigma, theta, type);
		g2Dot<Graph, EdgeIt, VertexIt>(g);
	}
	else { std::cerr << "Initial graph is not planar..." << std::endl; exit(-1); }

	gsl_rng_free(r);

  return 0;
}
