#ifndef __INDEX_H__
#define __INDEX_H__

#include <vector>

namespace std {

class index {
private:
	unsigned long _full, _sub;
	unsigned long _n;
	vector<unsigned long> _mask;
public:
	template<class _Container>
	index (const _Container& full, const _Container& sub) {
		_mask = vector<unsigned long>(full.size(), 0);
		typename _Container::const_iterator b = sub.begin();
		int i = 0, j = 0;
		for(typename _Container::const_iterator a = full.begin();
					a != full.end(); ++a, ++i) {
			for(; b != sub.end() && *b < *a; ++b, ++j);
			if(b != sub.end() && *b == *a) _mask[i] = 1l << j;
		}
		_n = 1l << full.size();
		_full = _sub = 0;
	};
	template<class _Container, class _InputIterator>
	index (const _Container& full,
				const _InputIterator first, const _InputIterator last) {
		_mask = vector<unsigned long>(full.size(), 0);
		_InputIterator b;
		int i = 0, j;
		for(typename _Container::const_iterator a = full.begin();
					a != full.end(); ++a, ++i) {
			for(b = first, j = 0; !(*a == *b) && b != last; ++b, ++j);
			if(b != last) _mask[i] = 1l << j;
		}
		_n = 1l << full.size();
		_full = _sub = 0;
	};
	inline void next () {
		--_n;
		int b = 0;
		for(unsigned long s = _n ^ (_n - 1); s >>= 1; ++b);
		_full ^= 1l << b;
		_sub ^= _mask[b];
	};
	inline int running () const { return(_n != 0); };
	inline unsigned long full () const { return(_full); };
	inline unsigned long sub () const { return(_sub); };
};

class index2 {
private:
	unsigned long _full, _sub1, _sub2;
	unsigned long _n;
	vector<unsigned long> _mask1, _mask2;
public:
	template<class _Container>
	index2 (const _Container& full,
				const _Container& sub1, const _Container& sub2) {
		_mask1 = vector<unsigned long>(full.size(), 0);
		_mask2 = vector<unsigned long>(full.size(), 0);
		typename _Container::const_iterator b1 = sub1.begin();
		typename _Container::const_iterator b2 = sub2.begin();
		int i = 0, j1 = 0, j2 = 0;
		for(typename _Container::const_iterator a = full.begin(); a != full.end(); ++a, ++i) {
			for(; b1 != sub1.end() && *b1 < *a; ++b1, ++j1);
			if(b1 != sub1.end() && *b1 == *a) _mask1[i] = 1l << j1;
			for(; b2 != sub2.end() && *b2 < *a; ++b2, ++j2);
			if(b2 != sub2.end() && *b2 == *a) _mask2[i] = 1l << j2;
		}
		_n = 1l << full.size();
		_full = _sub1 = _sub2 = 0;
	};
	inline void next () {
		--_n;
		int b = 0;
		for(unsigned long s = _n ^ (_n - 1); s >>= 1; ++b);
		_full ^= 1l << b;
		_sub1 ^= _mask1[b];
		_sub2 ^= _mask2[b];
	};
	inline int running () const { return(_n != 0); };
	inline unsigned long full () const { return(_full); };
	inline unsigned long sub1 () const { return(_sub1); };
	inline unsigned long sub2 () const { return(_sub2); };
};

}; // end of namespace std

#endif

