#ifndef __LOGD_H__
#define __LOGD_H__

#include <math.h>
#include <iostream>

class logd {
private:
	double _logz;
	int _sign;
public:
	logd () {};
	explicit logd (double z) {
		_logz = log(fabs(z));
		_sign = z < 0;
	}

	logd& operator += (const logd x) {
		if(x._logz != -HUGE_VAL) {
			if(_logz > x._logz) {
				double e = exp(x._logz - _logz);
				_logz += log1p(_sign ^ x._sign ? -e : e);
			} else {
				double e = exp(_logz - x._logz);
				_logz = x._logz + log1p(_sign ^ x._sign ? -e : e);
				_sign = x._sign;
			}
		}
		return(*this);
	}
	logd& operator -= (const logd x) {
		_sign = !_sign;
		*this += x;
		_sign = !_sign;
		return(*this);
	}
	logd& operator *= (const logd x) {
		_logz += x._logz;
		_sign ^= x._sign;
		return(*this);
	}
	logd& operator /= (const logd x) {
		_logz -= x._logz;
		_sign ^= x._sign;
		return(*this);
	}

	logd operator + (const logd& x) const {
		return(logd(*this) += x);
	}
	logd operator - (const logd& x) const {
		return(logd(*this) -= x);
	}
	logd operator * (const logd& x) const {
		return(logd(*this) *= x);
	}
	logd operator / (const logd& x) const {
		return(logd(*this) /= x);
	}

	int operator == (const logd x) const {
		return(_logz == x._logz && (_sign == x._sign || _logz == -HUGE_VAL));
	}
	int operator < (const logd x) const {
		return(!(*this == x) && (_sign > x._sign ||
				(_sign == x._sign && ((_logz < x._logz) ^ _sign))));
	}
	int operator > (const logd x) const {
		return(!(*this < x || *this == x));
	}
	int operator != (const logd x) const {
		return(!(*this == x));
	}
	int operator <= (const logd x) const {
		return(!(*this > x));
	}
	int operator >= (const logd x) const {
		return(!(*this < x));
	}

	operator double () const {
		return(_sign ? -exp(_logz) : exp(_logz));
	}
};

#endif

